#include <algorithm>
#include <iostream>
#include <numeric>
#include <cassert>

typedef long long llong;
const int MAXN = 100000 + 10;
const int INF  = 1e9;

int n, q;
struct SegmentTree
{
    struct Node
    {
        llong answer[2];
        llong pref;
        llong suff;
        int cnt;

        Node()
        {
            answer[0] = answer[1] = 0;
            pref = suff = 0;
            cnt = 0;
        }
    };

    Node tree[4*MAXN];
    Node combine(Node left, Node right)
    {
        Node res;
        if (left.cnt == 0)
        {
            res = right;
            res.answer[0] -= res.pref;
            res.pref = std::max(res.pref, left.pref);
            res.answer[0] += res.pref;
            if (right.cnt == 0)
            {
                res.suff = std::max(res.suff, left.suff);
            }

            return res;
        }

        if (right.cnt == 0)
        {
            res = left;
            res.answer[res.cnt & 1] -= res.suff;
            res.suff = std::max(res.suff, right.pref);
            res.answer[res.cnt & 1] += res.suff;
            return res;
        }

        res = left;
        res.cnt = left.cnt + right.cnt;
        res.pref = left.pref;
        res.suff = right.suff;

        right.answer[0] -= right.pref;
        res.answer[left.cnt & 1] -= left.suff;
        res.answer[left.cnt & 1] += std::max(left.suff, right.pref);
        res.answer[0] += right.answer[(left.cnt) & 1];
        res.answer[1] += right.answer[((left.cnt) & 1) ^ 1];
        return res;
    }

    void updateADD(int l, int r, int node, int queryPos, int queryADD)
    {
        if (l == r)
        {
            tree[node].answer[0] += queryADD;
            tree[node].pref += queryADD;
            tree[node].suff += queryADD;
            if (tree[node].cnt) tree[node].suff = 0;
            return;
        }

        int mid = (l + r) / 2;
        if (queryPos <= mid) updateADD(l, mid, 2*node, queryPos, queryADD);
        else updateADD(mid + 1, r, 2*node + 1, queryPos, queryADD);
        tree[node] = combine(tree[2*node], tree[2*node + 1]);
    }

    void updateXOR(int l, int r, int node, int queryPos)
    {
        if (l == r)
        {
            tree[node].cnt ^= 1;
            if (tree[node].cnt) tree[node].suff = 0;
            else tree[node].suff = tree[node].pref;
            return;
        }

        int mid = (l + r) / 2;
        if (queryPos <= mid) updateXOR(l, mid, 2*node, queryPos);
        else updateXOR(mid + 1, r, 2*node + 1, queryPos);
        tree[node] = combine(tree[2*node], tree[2*node + 1]);
    }

    Node query(int l, int r, int node, int queryL, int queryR)
    {
        if (queryL <= l && r <= queryR)
        {
            return tree[node];
        }

        Node res;
        int mid = (l + r) / 2;
        if (queryL <= mid) res = combine(res, query(l, mid, 2*node, queryL, queryR));
        if (mid + 1 <= queryR) res = combine(res, query(mid + 1, r, 2*node + 1, queryL, queryR));
        return res;
    }

    void updateADD(int pos, int val)
    {
        updateADD(1, n, 1, pos, val);
    }

    void updateXOR(int pos)
    {
        updateXOR(1, n, 1, pos);
    }

    llong query(int l, int r)
    {
        int realCNT = 0;
        if (l > 1) realCNT = query(1, n, 1, 1, l - 1).cnt;
        Node res = query(1, n, 1, l, r);
        return res.answer[((realCNT) & 1) ^ 1];
    }
};

SegmentTree tree;
void solve()
{
    for (int i = 1 ; i <= q ; ++i)
    {
        int qType;
        std::cin >> qType;

        if (qType == 3)
        {
            int pos, val;
            std::cin >> pos >> val;
            tree.updateADD(pos, val);
            continue;
        }

        if (qType == 2 || qType == 1)
        {
            int pos, val;
            std::cin >> pos;
            tree.updateXOR(pos);
            continue;
        }

        if (qType == 4)
        {
            int l, r;
            std::cin >> l >> r;
            std::cout << tree.query(l, r) << '\n';
            continue;
        }
    }
}

void input()
{
    std::cin >> n >> q;
}

void fastIOI()
{
    std::ios_base :: sync_with_stdio(0);
    std::cout.tie(nullptr);
    std::cin.tie(nullptr);
}

int main()
{
    fastIOI();
    input();
    solve();

    return 0;
}


/**
10 20
1 4
3 3 7
3 4 1
3 5 8
3 6 5
4 3 6
1 1
4 3 6
3 10 3
3 9 2
3 8 9
1 8
4 1 10
4 4 9
3 9 8
1 7
3 2 6
4 2 9
2 1
4 2 9

*/
