#include <algorithm>
#include <iostream>
#include <numeric>
#include <cassert>
#include <vector>
#include <set>

typedef long long llong;
const int MAXN = 100000 + 10;
const int INF  = 1e9;

struct Point
{
    llong x, y;
    int idx;

    friend bool operator < (const Point &a, const Point &b)
    {
        return a.x < b.x || (a.x == b.x && a.y > b.y);
    }
};

struct Line
{
    llong x, lY, rY;
    int type;

    friend bool operator < (const Line &a, const Line &b)
    {
        return a.x < b.x || (a.x == b.x && a.lY < b.lY);
    }
};


llong size(Point a, Point b, Point c)
{
    return a.x * b.y + a.y * c.x + b.x * c.y - b.y * c.x - a.x * c.y - a.y * b.x;
}

int n, n2, m;
Point real[MAXN];
Point p[MAXN];
Point p2[MAXN];
bool par[MAXN];
bool isActive[MAXN];
bool goodAngle[MAXN];
std::set <int> xVals;
std::vector <Line> lines;
bool isOK = true;

void add(int l, int r)
{
    for (int i = l ; i <= r ; ++i)
    {
        isActive[i] = true;
    }
}

void remove(int l, int r)
{
    for (int i = l ; i <= r ; ++i)
    {
        isActive[i] = false;
        if (par[i]) isOK = false;
        par[i] = 0;
    }
}

void checkOK()
{
    int cnt = 0;
    for (int i = 1 ; i <= 1000 ; ++i)
    {
        if (par[i])
        {
            cnt++;
        } else
        {
            if (cnt & 1)
            {
                isOK = false;
                break;
            }

            cnt = 0;
        }
    }

    if (cnt > 0 && (cnt & 1))
    {
        isOK = false;
    }
}

void addPAR()
{
    for (int i = 1 ; i <= 1000 ; ++i)
    {
        if (isActive[i])
        {
            par[i] ^= 1;
        }
    }
}

int check()
{
    int curr = -1;
    for (int i = 1 ; i <= 1000 ; ++i)
    {
        if (isActive[i])
        {
            if (curr == (par[i] ^ 1))
            {
                return -1;
            }

            curr = par[i];
        }
    }

    return curr;
}

void solve()
{
    /*p[++n2] = p[1];
    for (int i = 2 ; i <= n ; ++i)
    {
        int prev = i - 1;
        int next = i % n + 1;
        if ((p[prev].x != p[i].x || p[next].x != p[i].x) || (p[prev].y != p[i].y || p[next].y != p[i].y))
        {
            p2[++n2] = p[i];
        }
    }

    n = n2;
    for (int i = 1 ; i <= n ; ++i)
    {
        p[i] = p2[i];
        real[i] = p[i];
    }*/

    for (int i = 1 ; i <= n ; ++i)
    {
        int prev = i - 1;
        if (prev == 0) prev = n;
        int next = i % n + 1;

        if (p[i].x != p[next].x)
        {
            continue;
        }

        if ((size(p[prev], p[i], p[next]) > 0) ^ (p[i].x > p[prev].x))
        {
            lines.push_back({p[i].x, p[i].y, p[next].y, 1});
            if (p[i].y > p[next].y) std::swap(lines.back().lY, lines.back().rY);
        } else
        {
            lines.push_back({p[i].x, p[i].y, p[next].y, 0});
            if (p[i].y > p[next].y) std::swap(lines.back().lY, lines.back().rY);
        }
    }

    for (int i = 2 ; i < n ; ++i)
    {
        goodAngle[i] = (size(p[i - 1], p[i], p[i + 1]) > 0);
    }

    int ans = 0;
    int last = 0;
    goodAngle[1] = goodAngle[n] = true;

    for (int i = 1 ; i <= n ; ++i)
    {
        p[i].idx = i;
        xVals.insert(p[i].x);
    }

    int ptr = 0;
    std::sort(lines.begin(), lines.end());
    for (const int &x : xVals)
    {checkOK();
        if (!isOK) break;

        std::vector <Line> toADD;
        std::vector <Line> toDEL;

        while (ptr < lines.size() && lines[ptr].x <= x)
        {
            if (lines[ptr].type == 1) toADD.push_back(lines[ptr]);
            else toDEL.push_back(lines[ptr]);
            ptr++;
        }

        for (const Line &c : toDEL)
        {
            remove(c.lY + 1, c.rY);
        }

        if ((x - last) & 1)
        {
            addPAR();
        }

        int chk = check();
        for (const Line &c : toADD)
        {
            add(c.lY + 1, c.rY);
        }


        if (chk == -1) continue;
        ans = std::max(ans, x - chk);
        last = x;
    }

    std::cout << ans << '\n';
}

void input()
{
    std::cin >> n >> m;
    for (int i = 1 ; i <= n ; ++i)
    {
        std::cin >> p[i].x >> p[i].y;
    }

    if (p[1].x == p[2].x)
    {
        std::reverse(p + 2, p + 1 + n);
    }
}

void fastIOI()
{
    std::ios_base :: sync_with_stdio(0);
    std::cout.tie(nullptr);
    std::cin.tie(nullptr);
}

int main()
{
    fastIOI();
    input();

    if ((abs(p[n].y - p[1].y)) % 2 == 1)
    {
        std::cout << 0 << '\n';
        return 0;
    }

    int x = 0;
    for (int i = 2 ; i <= n ; ++i)
    {
        if (((p[i].x - p[i - 1].x) & 1) || ((p[1].y - p[i].y) & 1))
        {
            break;
        }

        x = p[i].x;
    }

    std::cout << x << '\n';
    // solve();

    return 0;
}


/**
10 20
1 4
3 3 7
3 4 1
3 5 8
3 6 5
4 3 6
1 1
4 3 6
3 10 3
3 9 2
3 8 9
1 8
4 1 10
4 4 9
3 9 8
1 7
3 2 6
4 2 9
2 1
4 2 9

*/
