#include <bits/stdc++.h>

using namespace std;

const int maxn = 100005;

int n, q;

long long a[maxn];
bool divider[maxn];

struct Node
{
    int cnt;
    int maxPreffix;
    int maxSuffix;
    int oddSum;
    int evenSum;
    bool isEnding;
};

struct SegmentTree
{
    Node tree[4 * maxn];

    Node combine(Node le, Node ri)
    {
        if (le.cnt == -1)
        {
            return ri;
        }

        if (ri.cnt == -1)
        {
            return le;
        }

        Node result;

        if (le.isEnding == true)
        {
            result.cnt = le.cnt + ri.cnt;

            if (le.cnt % 2 == 1)
            {
                result.oddSum = le.oddSum + ri.evenSum;
                result.evenSum = le.evenSum + ri.oddSum;
            }
            else
            {
                result.oddSum = le.oddSum + ri.oddSum;
                result.evenSum = le.evenSum + ri.evenSum;
            }

            result.isEnding = ri.isEnding;
            result.maxPreffix = le.maxPreffix;
            result.maxSuffix = ri.maxSuffix;
        }
        else
        {
            result.cnt = le.cnt + ri.cnt;

            if (le.cnt % 2 == 1)
            {
                result.oddSum = le.oddSum + ri.evenSum - le.maxSuffix - ri.maxPreffix + max(le.maxSuffix, ri.maxPreffix);
                result.evenSum = le.evenSum + ri.oddSum;
            }
            else
            {
                result.oddSum = le.oddSum + ri.oddSum;
                result.evenSum = le.evenSum + ri.evenSum - le.maxSuffix - ri.maxPreffix + max(le.maxSuffix, ri.maxPreffix);
            }

            result.isEnding = ri.isEnding;

            result.maxPreffix = le.maxPreffix;

            if (le.cnt == 0)
            {
                result.maxPreffix = max(result.maxPreffix, ri.maxPreffix);
            }

            result.maxSuffix = ri.maxSuffix;

            if (ri.cnt == 0)
            {
                result.maxSuffix = max(result.maxSuffix, le.maxSuffix);
            }
        }

        return result;
    }

    void update(int node, int l, int r, int pos)
    {
        if (l == r)
        {
            tree[node].isEnding = divider[l];
            tree[node].cnt = divider[l];

            if (tree[node].isEnding == 0)
            {
                tree[node].maxSuffix = a[l];
            }
            else
            {
                tree[node].maxSuffix = 0;
            }

            tree[node].maxPreffix = a[l];
            tree[node].evenSum = a[l];
            tree[node].oddSum = 0;

            return;
        }

        int mid = (l + r) / 2;

        if (pos <= mid)
        {
            update(2 * node, l, mid, pos);
        }
        else
        {
            update(2 * node + 1, mid + 1, r, pos);
        }

        tree[node] = combine(tree[2 * node], tree[2 * node + 1]);
    }

    Node query(int node, int l, int r, int ql, int qr)
    {
        if (ql <= l && r <= qr)
        {
            return tree[node];
        }

        int mid = (l + r) / 2;

        Node result;
        result.cnt = -1;

        if (ql <= mid)
        {
            result = combine(query(2 * node, l, mid, ql, qr), result);
        }

        if (qr >= mid + 1)
        {
            result = combine(result, query(2 * node + 1, mid + 1, r, ql, qr));
        }

        return result;
    }

    void update(int pos)
    {
        update(1, 1, n, pos);
    }

    Node query(int l, int r)
    {
        return query(1, 1, n, l, r);
    }
};

SegmentTree tree;

struct FenwickTree
{
    int tree[maxn];

    void update(int pos, int val)
    {
        for (int i = pos; i <= n; i += (i & (-i)))
        {
            tree[i] += val;
        }
    }

    int query(int pos)
    {
        int result = 0;

        for (int i = pos; i >= 1; i -= (i & (-i)))
        {
            result += tree[i];
        }

        return result;
    }
};

FenwickTree bit;

void fastIO()
{
    ios_base::sync_with_stdio(0);
    cin.tie(0);
    cout.tie(0);
}

int main()
{
    fastIO();

    cin >> n >> q;

    for (int i = 1; i <= q; ++i)
    {
        int type;
        cin >> type;

        if (type == 1)
        {
            int pos;
            cin >> pos;

            divider[pos] = true;
            tree.update(pos);
            bit.update(pos, +1);
        }
        else if (type == 2)
        {
            int pos;
            cin >> pos;

            divider[pos] = false;
            tree.update(pos);
            bit.update(pos, -1);
        }
        else if (type == 3)
        {
            int pos, delta;
            cin >> pos >> delta;

            a[pos] += delta;
            tree.update(pos);
        }
        else
        {
            int l, r;
            cin >> l >> r;

            Node result = tree.query(l, r);

            //cout << result.cnt << ' ' << result.oddSum << ' ' << result.evenSum << ' ' << result.maxPreffix << ' ' << result.maxSuffix << ' ' << result.isEnding << "\n";

            if (bit.query(l - 1) % 2 == 1)
            {
                cout << result.evenSum << "\n";
            }
            else
            {
                cout << result.oddSum << "\n";
            }
        }
    }

    return 0;
}

/*
10 8
1 4
3 3 7
3 4 1
3 5 8
3 6 5
4 3 6
1 1
4 3 6
*/
